realTerrainHotfix = {};
realTerrainHotfix.g_currentModDirectory = g_currentModDirectory;
addModEventListener(realTerrainHotfix);

function realTerrainHotfix:loadMap(name)
	if not g_currentMission.realTerrainModHotfixIsLoaded then
		if SpecializationUtil.specializations["realTerrain"] ~= nil then
			realTerrainHotfix:fixDraw();
		end;

		g_currentMission.realTerrainModHotfixIsLoaded = true;
	else
		print("realTerrainHotfix - Error: The Real Terrain Hotfix mod have been loaded already! remove one of the copy's");
	end;
end;

function realTerrainHotfix:deleteMap()
	g_currentMission.realTerrainModHotfixIsLoaded = nil;
end;

function realTerrainHotfix:mouseEvent(posX, posY, isDown, isUp, button)
end;

function realTerrainHotfix:keyEvent(unicode, sym, modifier, isDown)
end;

function realTerrainHotfix:update(dt)
end;

function realTerrainHotfix:draw()
end;

function realTerrainHotfix:fixDraw()
	local realTerrain = SpecializationUtil.getSpecialization("realTerrain");
	if realTerrain.update then
		realTerrain.update = function (self, dt)
			-- We only need to run this code on the server.
			if self.isServer and self:getIsActive() and self.wheels and #self.wheels > 0 then
				--if realTerrain.printWheelInfo then
				--	print(realTerrainManager:tableShow(self.wheels, "self.wheels"));
				--	realTerrain.printWheelInfo = false;
				--end;

				-- Disable driveControl groundResponse if active. It will mess up our script.
				if g_currentMission.driveControl ~= nil and g_currentMission.driveControl.useModules.groundResponse then
					g_currentMission.driveControl.useModules.groundResponse = false;
					print("realTerrain: Disabling driveControl module groundResponse.");
				end;

				-- Check if vehicle has an plough attached if isDirty
				if self.realTerrain.isDirty then
					self.realTerrain.isDirty = false;
					self.realTerrain.hasPloughAttached = realTerrain.hasPloughAttached(self);
				end;

				-- Only update if we are moving
				if self.movingDirection ~= 0 then
					-- Check if only one side is on ploughed areas.
					local ExtraDrop = false;
					local leftSide = false;
					local rightSide = false;
					for id, wheel in ipairs(self.wheels) do
						local wx, wy, wz = getWheelShapeContactPoint(wheel.node, wheel.wheelShape);
						if wx ~= nil then
							local density = getDensityAtWorldPos(g_currentMission.terrainDetailId, wx, wz) % 16;
							if density == 2^g_currentMission.ploughChannel then
								if wheel.RT.isLeftWheel then
									leftSide = true;
								elseif wheel.RT.isRightWheel then
									rightSide = true;
								end;
							end;
						end;
					end;
					if leftSide ~= rightSide then
						ExtraDrop = true;
					end;

					-- Update each wheel
					for id, wheel in ipairs(self.wheels) do
						-- Make sure we have this value already set befure using it in a check.
						if wheel.RT.currentDensity == nil then
							wheel.RT.currentDensity = 0;
						end;

						-- Get the world position of the wheel.
						local wx, wy, wz = getWheelShapeContactPoint(wheel.node, wheel.wheelShape);
						if wx ~= nil then
							local distance = Utils.vector3Length(wx - wheel.RT.lastX, wy - wheel.RT.lastY, wz - wheel.RT.lastZ);
							local density = getDensityAtWorldPos(g_currentMission.terrainDetailId, wx, wz) % 16;

							local isField = (density) > 0;
							local groundType = WheelsUtil.getGroundType(isField,false,wheel.lastColor)

							local needUpdate = distance >= wheel.RT.travelDistToUpdate or wheel.RT.currentDensity ~= density or self.realTerrain.extraDrop ~= ExtraDrop;

							----------------------------------------------------------------
							-- Change wheel radius if density have changed
							----------------------------------------------------------------
							if groundType == WheelsUtil.GROUND_FIELD then
								if wheel.RT.travelDistToUpdate == 10000 or needUpdate then
									if self.realTerrain.hasPloughingSpec and density ==(2^g_currentMission.ploughChannel) then
										-- Disable realTerrain on ploughChannel density if the vehicle has hasPloughingSpec
										realTerrain:resetWheelInfo(wheel, density);
									else
										local terrainInfo = self.realTerrain.terrainType[groundType][density];

										-- Set the radius Delta based on density level
										if terrainInfo then
											dropDis = terrainInfo.defDropDis;
											minBumpHeight = terrainInfo.minBumpHeight;
											maxBumpHeight = terrainInfo.maxBumpHeight;

											if self.realTerrain.hasPloughAttached and ExtraDrop then
												if density == 2^g_currentMission.ploughChannel then
													-- make it drop extra, but less bumpy
													wheel.RT.extraDrop = true;
													minBumpHeight = minBumpHeight * 0.1;
													maxBumpHeight = maxBumpHeight * 0.1;
												end;
												dropDis = dropDis + terrainInfo.leftRightDropDis;
											else
												wheel.RT.extraDrop = false;
											end;


											realTerrain:updateWheelInfo(groundType, wheel, density, needUpdate, dropDis, minBumpHeight, maxBumpHeight);
											-- No Density (Left in here just to be sure.)
										else
											realTerrain:resetWheelInfo(wheel);
										end;
									end;

									-- Update the saved position
									wheel.RT.lastX = wx;
									wheel.RT.lastY = wy;
									wheel.RT.lastZ = wz;

									if (wheel.RT.currentDensity ~= 2^g_currentMission.ploughChannel and wheel.RT.extraDrop)
											or (wheel.RT.currentDensity ~= density and wheel.RT.currentDensity == 2^g_currentMission.ploughChannel)
											or (wheel.RT.currentDensity == 2^g_currentMission.ploughChannel and self.realTerrain.extraDrop ~= ExtraDrop and (not ExtraDrop or wheel.RT.extraDrop)) then
										wheel.RT.fastUpdate = true;
									else
										wheel.RT.fastUpdate = false;
									end;

									-- Make sure we update to current density.
									wheel.RT.currentDensity = density;

								end;

								----------------------------------------------------------------
								-- Change wheel radius based on Soft terrain (Grass and dirt roads)
								----------------------------------------------------------------
							elseif groundType == WheelsUtil.GROUND_SOFT_TERRAIN then
								if wheel.RT.travelDistToUpdate == 10000 or needUpdate then
									local terrainInfo = self.realTerrain.terrainType[groundType];

									-- Set the radius Delta based on density level
									realTerrain:updateWheelInfo(groundType, wheel, density, needUpdate, terrainInfo.defDropDis, terrainInfo.minBumpHeight, terrainInfo.maxBumpHeight); -- Values needs tweaking.

									-- Update the saved position
									wheel.RT.lastX = wx;
									wheel.RT.lastY = wy;
									wheel.RT.lastZ = wz;

									if wheel.RT.currentDensity == 2^g_currentMission.ploughChannel and wheel.RT.extraDrop then
										wheel.RT.fastUpdate = true;
									else
										wheel.RT.fastUpdate = false;
									end;
									wheel.RT.extraDrop = false;

									wheel.RT.currentDensity = density;
								end;
							else
								realTerrain:resetWheelInfo(wheel);
							end;

							-- Update wheel
							if wheel.RT.currentRadiusDelta ~= wheel.RT.radiusDelta then
								-- Get current speed in km/h
								local currentSpeed = self.lastSpeedReal * 3600;
								-- Make speed multiplier (0 - 1 based on 0-80 km/h)
								local speedMultiplier = math.min(0.0125 * currentSpeed, 1);

								-- need to update faster if needed
								if wheel.RT.fastUpdate then
									speedMultiplier = speedMultiplier * 10;

									-- Need to update slower if on Soft terrain
								elseif groundType == WheelsUtil.GROUND_SOFT_TERRAIN and not wheel.RT.fastUpdate then
									speedMultiplier = speedMultiplier * 0.3;
								end;

								-- Make more smooth transition on bumps.
								if wheel.RT.currentRadiusDelta < wheel.RT.radiusDelta then
									wheel.RT.currentRadiusDelta = math.min(wheel.RT.radiusDelta, wheel.RT.currentRadiusDelta + ((dt / 1000) * speedMultiplier));
								else
									wheel.RT.currentRadiusDelta = math.max(wheel.RT.radiusDelta, wheel.RT.currentRadiusDelta - ((dt / 1000) * speedMultiplier));
								end;

								local collisionMask = 255 - 4; -- all up to bit 8, except bit 2 which is set by the players kinematic object
								local positionY = wheel.netInfo.yMin + wheel.netInfo.yRange;
								createWheelShape(wheel.node, wheel.positionX, positionY, wheel.positionZ, wheel.RT.origWheelRadius - wheel.RT.currentRadiusDelta, wheel.suspTravel, wheel.spring, wheel.damper, wheel.mass, collisionMask, wheel.wheelShape);

								-- Change wheel radius, so the track is placed in the right height.
								local newRadius = wheel.RT.origWheelRadius - wheel.RT.currentRadiusDelta
								if wheel.RT.oldWheelRadius ~= newRadius then
									wheel.radius = newRadius;
									wheel.RT.oldWheelRadius = wheel.radius;
									self.realTerrain.networkUpdate = true;
								end;
							end;
						else
							realTerrain:resetWheelInfo(wheel);
						end;
					end;

					self.realTerrain.extraDrop = ExtraDrop;
				end;
			end;
		end;
		print("Real Terrain Hotfix: Fixed ploughingSpec Compatibility");
	end;
end;